@@ -9,7 +9,7 @@ from os.path import join
 
 import pytest
 
-from ..travis import sync_from_log, determine_push_rights
+from ..travis import sync_from_log, determine_push_rights, copy_to_tmp
 
 @pytest.mark.parametrize("src", ["src"])
 @pytest.mark.parametrize("dst", ['.', 'dst'])
@@ -56,6 +56,34 @@ def test_sync_from_log(src, dst):
             with open(join(src, 'test3'), 'w') as f:
                 f.write('test3')
 
+            # First test it is ignored when excluded
+            added, removed = sync_from_log(src, dst, 'logfile',
+                exclude=['test3'])
+
+            assert added == [
+                join(dst, 'test1'),
+                join(dst, 'testdir', 'test2'),
+                'logfile',
+            ]
+
+
+            assert removed == []
+
+            with open(join(dst, 'test1')) as f:
+                assert f.read() == 'test1'
+
+            with open(join(dst, 'testdir', 'test2')) as f:
+                assert f.read() == 'test2'
+
+            assert not os.path.exists(join(dst, 'test3'))
+
+            with open('logfile') as f:
+                assert f.read() == '\n'.join([
+                    join(dst, 'test1'),
+                    join(dst, 'testdir', 'test2'),
+                    ])
+
+            # Now test it it added normally
             added, removed = sync_from_log(src, dst, 'logfile')
 
             assert added == [
@@ -86,6 +114,36 @@ def test_sync_from_log(src, dst):
             # Delete a file
             os.remove(join(src, 'test3'))
 
+            # First test it is ignored with exclude
+            added, removed = sync_from_log(src, dst, 'logfile', exclude=['test3'])
+            assert added == [
+                join(dst, 'test1'),
+                join(dst, 'testdir', 'test2'),
+                'logfile',
+            ]
+
+            assert removed == []
+
+            with open(join(dst, 'test1')) as f:
+                assert f.read() == 'test1'
+
+            with open(join(dst, 'testdir', 'test2')) as f:
+                assert f.read() == 'test2'
+
+            with open(join(dst, 'test3')) as f:
+                assert f.read() == 'test3'
+
+            with open('logfile') as f:
+                assert f.read() == '\n'.join([
+                    join(dst, 'test1'),
+                    join(dst, 'testdir', 'test2'),
+                    ])
+
+            # Then test it is removed normally
+            # (readd it to the log file, since the exclude removed it)
+            with open('logfile', 'a') as f:
+                f.write('\n' + join(dst, 'test3'))
+
             added, removed = sync_from_log(src, dst, 'logfile')
 
             assert added == [
@@ -140,6 +198,25 @@ def test_sync_from_log(src, dst):
                     join(dst, 'testdir', 'test2'),
                     ])
 
+            # Test excluding a directory
+
+            os.makedirs(join(src, 'testdir2'))
+            with open(join(src, 'testdir2', 'test2'), 'w') as f:
+                f.write('test2')
+
+            added, removed = sync_from_log(src, dst, 'logfile', exclude=['testdir2'])
+
+
+            assert added == [
+                join(dst, 'test1'),
+                join(dst, 'testdir', 'test2'),
+                'logfile',
+            ]
+
+            assert removed == []
+
+            assert not os.path.exists(join(dst, 'testdir2'))
+
         finally:
             os.chdir(old_curdir)
 
@@ -192,3 +269,22 @@ def test_determine_push_rights(branch_whitelist, TRAVIS_BRANCH,
         TRAVIS_PULL_REQUEST=TRAVIS_PULL_REQUEST,
         TRAVIS_TAG=TRAVIS_TAG,
         build_tags=build_tags) == canpush
+
+@pytest.mark.parametrize("src", ["src", "."])
+def test_copy_to_tmp(src):
+    with tempfile.TemporaryDirectory() as dir:
+        os.makedirs(os.path.join(dir, src), exist_ok=True)
+        with open(os.path.join(dir, src, "test"), 'w') as f:
+            f.write('test')
+
+        new_dir = copy_to_tmp(os.path.join(dir, src))
+
+        assert os.path.exists(new_dir)
+        with open(os.path.join(new_dir, 'test'), 'r') as f:
+            assert f.read() == 'test'
+
+        new_dir2 = copy_to_tmp(os.path.join(dir, src, 'test'))
+
+        assert os.path.exists(new_dir2)
+        with open(os.path.join(new_dir2), 'r') as f:
+            assert f.read() == 'test'
